from abc import ABC, abstractmethod
from typing import Dict, Any

from ai21.models.responses.gec_response import GECResponse


class GEC(ABC):
    _module_name = "gec"

    @abstractmethod
    def create(self, text: str, **kwargs) -> GECResponse:
        pass

    def _json_to_response(self, json: Dict[str, Any]) -> GECResponse:
        return GECResponse.from_dict(json)

    def _create_body(self, text: str) -> Dict[str, Any]:
        return {"text": text}



from abc import ABC, abstractmethod
from typing import List, Any, Dict, Optional

from ai21.models.chat_message import ChatMessage
from ai21.models.penalty import Penalty
from ai21.models.responses.chat_response import ChatResponse


class Chat(ABC):
    _module_name = "chat"

    @abstractmethod
    def create(
        self,
        model: str,
        messages: List[ChatMessage],
        system: str,
        *,
        num_results: Optional[int] = 1,
        temperature: Optional[float] = 0.7,
        max_tokens: Optional[int] = 300,
        min_tokens: Optional[int] = 0,
        top_p: Optional[float] = 1.0,
        top_k_return: Optional[int] = 0,
        stop_sequences: Optional[List[str]] = None,
        frequency_penalty: Optional[Penalty] = None,
        presence_penalty: Optional[Penalty] = None,
        count_penalty: Optional[Penalty] = None,
        **kwargs,
    ) -> ChatResponse:
        pass

    def _json_to_response(self, json: Dict[str, Any]) -> ChatResponse:
        return ChatResponse.from_dict(json)

    def _create_body(
        self,
        model: str,
        messages: List[ChatMessage],
        system: str,
        num_results: Optional[int] = 1,
        temperature: Optional[float] = 0.7,
        max_tokens: Optional[int] = 300,
        min_tokens: Optional[int] = 0,
        top_p: Optional[float] = 1.0,
        top_k_return: Optional[int] = 0,
        stop_sequences: Optional[List[str]] = None,
        frequency_penalty: Optional[Penalty] = None,
        presence_penalty: Optional[Penalty] = None,
        count_penalty: Optional[Penalty] = None,
    ) -> Dict[str, Any]:
        return {
            "model": model,
            "system": system,
            "messages": [message.to_dict() for message in messages],
            "temperature": temperature,
            "maxTokens": max_tokens,
            "minTokens": min_tokens,
            "numResults": num_results,
            "topP": top_p,
            "topKReturn": top_k_return,
            "stopSequences": stop_sequences,
            "frequencyPenalty": None if frequency_penalty is None else frequency_penalty.to_dict(),
            "presencePenalty": None if presence_penalty is None else presence_penalty.to_dict(),
            "countPenalty": None if count_penalty is None else count_penalty.to_dict(),
        }



from enum import Enum


class ImprovementType(str, Enum):
    FLUENCY = "fluency"
    VOCABULARY_SPECIFICITY = "vocabulary/specificity"
    VOCABULARY_VARIETY = "vocabulary/variety"
    CLARITY_SHORT_SENTENCES = "clarity/short-sentences"
    CLARITY_CONCISENESS = "clarity/conciseness"



from dataclasses import dataclass
from typing import List, Optional

from ai21.models.ai21_base_model_mixin import AI21BaseModelMixin


@dataclass
class Highlight(AI21BaseModelMixin):
    text: str
    start_index: int
    end_index: int


@dataclass
class SegmentSummary(AI21BaseModelMixin):
    summary: Optional[str]
    segment_text: str
    segment_html: Optional[str]
    segment_type: str
    has_summary: bool
    highlights: List[Highlight]


@dataclass
class SummarizeBySegmentResponse(AI21BaseModelMixin):
    segments: List[SegmentSummary]



from typing import Optional, List

from ai21.clients.common.dataset_base import Dataset
from ai21.models.responses.dataset_response import DatasetResponse
from ai21.clients.studio.resources.studio_resource import StudioResource


class StudioDataset(StudioResource, Dataset):
    def create(
        self,
        file_path: str,
        dataset_name: str,
        *,
        selected_columns: Optional[str] = None,
        approve_whitespace_correction: Optional[bool] = None,
        delete_long_rows: Optional[bool] = None,
        split_ratio: Optional[float] = None,
        **kwargs,
    ):
        files = {"dataset_file": open(file_path, "rb")}
        body = self._create_body(
            dataset_name=dataset_name,
            selected_columns=selected_columns,
            approve_whitespace_correction=approve_whitespace_correction,
            delete_long_rows=delete_long_rows,
            split_ratio=split_ratio,
        )
        return self._post(
            url=self._base_url(),
            body=body,
            files=files,
        )

    def list(self) -> List[DatasetResponse]:
        response = self._get(url=self._base_url())
        return [self._json_to_response(r) for r in response]

    def get(self, dataset_pid: str) -> DatasetResponse:
        url = f"{self._base_url()}/{dataset_pid}"
        response = self._get(url=url)

        return self._json_to_response(response)

    def _base_url(self) -> str:
        return f"{self._client.get_base_url()}/{self._module_name}"



from dataclasses import dataclass
from typing import List, Optional

from ai21.models.ai21_base_model_mixin import AI21BaseModelMixin


@dataclass
class SourceDocument(AI21BaseModelMixin):
    field_id: str
    name: str
    highlights: List[str]
    public_url: Optional[str] = None


@dataclass
class LibraryAnswerResponse(AI21BaseModelMixin):
    id: str
    answer_in_context: bool
    answer: Optional[str] = None
    sources: Optional[List[SourceDocument]] = None



from ai21.clients.common.segmentation_base import Segmentation
from ai21.clients.studio.resources.studio_resource import StudioResource
from ai21.models.document_type import DocumentType
from ai21.models.responses.segmentation_response import SegmentationResponse


class StudioSegmentation(StudioResource, Segmentation):
    def create(self, source: str, source_type: DocumentType, **kwargs) -> SegmentationResponse:
        body = self._create_body(source=source, source_type=source_type)
        url = f"{self._client.get_base_url()}/{self._module_name}"
        raw_response = self._post(url=url, body=body)

        return self._json_to_response(raw_response)



from dataclasses import dataclass
from enum import Enum
from typing import List

from ai21.models.ai21_base_model_mixin import AI21BaseModelMixin


class CorrectionType(str, Enum):
    GRAMMAR = "Grammar"
    MISSING_WORD = "Missing Word"
    PUNCTUATION = "Punctuation"
    SPELLING = "Spelling"
    WORD_REPETITION = "Word Repetition"
    WRONG_WORD = "Wrong Word"


@dataclass
class Correction(AI21BaseModelMixin):
    suggestion: str
    start_index: int
    end_index: int
    original_text: str
    correction_type: CorrectionType


@dataclass
class GECResponse(AI21BaseModelMixin):
    id: str
    corrections: List[Correction]



import logging

from ai21.ai21_env_config import AI21EnvConfig

logger = logging.getLogger("ai21")


def _basic_config() -> None:
    logging.basicConfig(
        format="[%(asctime)s - %(name)s - %(levelname)s] %(message)s",
        datefmt="%Y-%m-%d %H:%M:%S",
    )


def setup_logger() -> None:
    _basic_config()

    if AI21EnvConfig.log_level == "debug":
        logger.setLevel(logging.DEBUG)
    elif AI21EnvConfig.log_level == "info":
        logger.setLevel(logging.INFO)



from typing import Optional

from ai21_tokenizer import Tokenizer

from ai21.tokenizers.ai21_tokenizer import AI21Tokenizer

_cached_tokenizer: Optional[AI21Tokenizer] = None


def get_tokenizer() -> AI21Tokenizer:
    """
    Get the tokenizer instance.

    If the tokenizer instance is not cached, it will be created using the Tokenizer.get_tokenizer() method.
    """
    global _cached_tokenizer

    if _cached_tokenizer is None:
        _cached_tokenizer = AI21Tokenizer(Tokenizer.get_tokenizer())

    return _cached_tokenizer



from dataclasses import dataclass
from typing import Optional, List

from ai21.models.ai21_base_model_mixin import AI21BaseModelMixin


@dataclass
class LibrarySearchResult(AI21BaseModelMixin):
    text: str
    file_id: str
    file_name: str
    score: float
    public_url: Optional[str] = None
    labels: Optional[List[str]] = None


@dataclass
class LibrarySearchResponse(AI21BaseModelMixin):
    id: str
    results: List[LibrarySearchResult]



from typing import Any

from ai21.ai21_env_config import AI21EnvConfig
from ai21.clients.studio.ai21_client import AI21Client
from ai21.errors import (
    AI21APIError,
    APITimeoutError,
    MissingApiKeyError,
    ModelPackageDoesntExistError,
    AI21Error,
    TooManyRequestsError,
)
from ai21.logger import setup_logger
from ai21.services.sagemaker import SageMaker
from ai21.version import VERSION

__version__ = VERSION
setup_logger()


def _import_bedrock_client():
    from ai21.clients.bedrock.ai21_bedrock_client import AI21BedrockClient

    return AI21BedrockClient


def _import_sagemaker_client():
    from ai21.clients.sagemaker.ai21_sagemaker_client import AI21SageMakerClient

    return AI21SageMakerClient


def _import_bedrock_model_id():
    from ai21.clients.bedrock.bedrock_model_id import BedrockModelID

    return BedrockModelID


def __getattr__(name: str) -> Any:
    try:
        if name == "AI21BedrockClient":
            return _import_bedrock_client()

        if name == "AI21SageMakerClient":
            return _import_sagemaker_client()

        if name == "BedrockModelID":
            return _import_bedrock_model_id()
    except ImportError as e:
        raise ImportError(f'Please install "ai21[AWS]" in order to use {name}') from e


__all__ = [
    "AI21EnvConfig",
    "AI21Client",
    "AI21APIError",
    "APITimeoutError",
    "AI21Error",
    "MissingApiKeyError",
    "ModelPackageDoesntExistError",
    "TooManyRequestsError",
    "AI21BedrockClient",
    "AI21SageMakerClient",
    "BedrockModelID",
    "SageMaker",
]



from typing import Optional, List

from ai21.resources import Penalty
from ai21.clients.bedrock.resources.bedrock_resource import BedrockResource
from ai21.models.responses.completion_response import CompletionsResponse


class BedrockCompletion(BedrockResource):
    def create(
        self,
        model_id: str,
        prompt: str,
        *,
        max_tokens: Optional[int] = None,
        num_results: Optional[int] = 1,
        min_tokens: Optional[int] = 0,
        temperature: Optional[float] = 0.7,
        top_p: Optional[int] = 1,
        top_k_return: Optional[int] = 0,
        stop_sequences: Optional[List[str]] = None,
        frequency_penalty: Optional[Penalty] = None,
        presence_penalty: Optional[Penalty] = None,
        count_penalty: Optional[Penalty] = None,
        **kwargs,
    ) -> CompletionsResponse:
        body = {
            "prompt": prompt,
            "maxTokens": max_tokens,
            "numResults": num_results,
            "minTokens": min_tokens,
            "temperature": temperature,
            "topP": top_p,
            "topKReturn": top_k_return,
            "stopSequences": stop_sequences or [],
            "frequencyPenalty": None if frequency_penalty is None else frequency_penalty.to_dict(),
            "presencePenalty": None if presence_penalty is None else presence_penalty.to_dict(),
            "countPenalty": None if count_penalty is None else count_penalty.to_dict(),
        }
        raw_response = self._invoke(model_id=model_id, body=body)

        return CompletionsResponse.from_dict(raw_response)



from dataclasses import dataclass
from typing import List, Optional, Union, Any, Dict

from ai21.models.ai21_base_model_mixin import AI21BaseModelMixin


@dataclass
class Prompt(AI21BaseModelMixin):
    text: Optional[str]
    tokens: Optional[List[Dict[str, Any]]] = None


@dataclass
class CompletionData(AI21BaseModelMixin):
    text: Optional[str]
    tokens: Optional[List[Dict[str, Any]]] = None


@dataclass
class CompletionFinishReason(AI21BaseModelMixin):
    reason: Optional[str] = None
    length: Optional[int] = None


@dataclass
class Completion(AI21BaseModelMixin):
    data: CompletionData
    finish_reason: CompletionFinishReason


@dataclass
class CompletionsResponse(AI21BaseModelMixin):
    id: Union[int, str]
    prompt: Prompt
    completions: List[Completion]



from typing import Optional

from ai21.clients.common.paraphrase_base import Paraphrase
from ai21.models.paraphrase_style_type import ParaphraseStyleType
from ai21.models.responses.paraphrase_response import ParaphraseResponse
from ai21.clients.sagemaker.resources.sagemaker_resource import SageMakerResource


class SageMakerParaphrase(SageMakerResource, Paraphrase):
    def create(
        self,
        text: str,
        *,
        style: Optional[ParaphraseStyleType] = None,
        start_index: Optional[int] = 0,
        end_index: Optional[int] = None,
        **kwargs,
    ) -> ParaphraseResponse:
        body = self._create_body(
            text=text,
            style=style,
            start_index=start_index,
            end_index=end_index,
        )
        response = self._invoke(body=body)

        return self._json_to_response(response)



from abc import ABC, abstractmethod
from typing import Optional, Any, Dict

from ai21.models.responses.summarize_response import SummarizeResponse
from ai21.models.summary_method import SummaryMethod


class Summarize(ABC):
    @abstractmethod
    def create(
        self,
        source: str,
        source_type: str,
        *,
        focus: Optional[str] = None,
        summary_method: Optional[SummaryMethod] = None,
        **kwargs,
    ) -> SummarizeResponse:
        pass

    def _json_to_response(self, json: Dict[str, Any]) -> SummarizeResponse:
        return SummarizeResponse.from_dict(json)

    def _create_body(
        self,
        source: str,
        source_type: str,
        focus: Optional[str],
        summary_method: Optional[str],
    ) -> Dict[str, Any]:
        return {
            "source": source,
            "sourceType": source_type,
            "focus": focus,
            "summaryMethod": summary_method,
        }



from dataclasses import dataclass

from ai21.models.ai21_base_model_mixin import AI21BaseModelMixin
from ai21.models.role_type import RoleType


@dataclass
class ChatMessage(AI21BaseModelMixin):
    role: RoleType
    text: str



from dataclasses import dataclass
from typing import Optional

from ai21.models.ai21_base_model_mixin import AI21BaseModelMixin


@dataclass
class AnswerResponse(AI21BaseModelMixin):
    id: str
    answer_in_context: Optional[bool] = None
    answer: Optional[str] = None



import json
from abc import ABC
from typing import Any, Dict

from ai21.clients.bedrock.bedrock_session import BedrockSession


class BedrockResource(ABC):
    def __init__(self, bedrock_session: BedrockSession):
        self._bedrock_session = bedrock_session

    def _invoke(self, model_id: str, body: Dict[str, Any]) -> Dict[str, Any]:
        return self._bedrock_session.invoke_model(
            input_json=json.dumps(body),
            model_id=model_id,
        )



from typing import Optional

from ai21.clients.common.answer_base import Answer
from ai21.models import AnswerLength, Mode
from ai21.models.responses.answer_response import AnswerResponse
from ai21.clients.studio.resources.studio_resource import StudioResource


class StudioAnswer(StudioResource, Answer):
    def create(
        self,
        context: str,
        question: str,
        *,
        answer_length: Optional[AnswerLength] = None,
        mode: Optional[Mode] = None,
        **kwargs,
    ) -> AnswerResponse:
        url = f"{self._client.get_base_url()}/{self._module_name}"

        body = self._create_body(context=context, question=question, answer_length=answer_length, mode=mode)

        response = self._post(url=url, body=body)

        return self._json_to_response(response)



from __future__ import annotations
import os
from dataclasses import dataclass
from typing import Optional

from ai21.constants import DEFAULT_API_VERSION, STUDIO_HOST


@dataclass(frozen=True)
class _AI21EnvConfig:
    api_key: Optional[str] = None
    api_version: str = DEFAULT_API_VERSION
    api_host: str = STUDIO_HOST
    timeout_sec: Optional[int] = None
    num_retries: Optional[int] = None
    aws_region: Optional[str] = None
    log_level: Optional[str] = None

    @classmethod
    def from_env(cls) -> _AI21EnvConfig:
        return cls(
            api_key=os.getenv("AI21_API_KEY"),
            api_version=os.getenv("AI21_API_VERSION", DEFAULT_API_VERSION),
            api_host=os.getenv("AI21_API_HOST", STUDIO_HOST),
            timeout_sec=os.getenv("AI21_TIMEOUT_SEC"),
            num_retries=os.getenv("AI21_NUM_RETRIES"),
            aws_region=os.getenv("AI21_AWS_REGION", "us-east-1"),
            log_level=os.getenv("AI21_LOG_LEVEL", "info"),
        )


AI21EnvConfig = _AI21EnvConfig.from_env()



from typing import Optional, List

from ai21.ai21_http_client import AI21HTTPClient
from ai21.models import Mode, AnswerLength
from ai21.models.responses.file_response import FileResponse
from ai21.models.responses.library_answer_response import LibraryAnswerResponse
from ai21.models.responses.library_search_response import LibrarySearchResponse
from ai21.clients.studio.resources.studio_resource import StudioResource


class StudioLibrary(StudioResource):
    _module_name = "library/files"

    def __init__(self, client: AI21HTTPClient):
        super().__init__(client)
        self.files = LibraryFiles(client)
        self.search = LibrarySearch(client)
        self.answer = LibraryAnswer(client)


class LibraryFiles(StudioResource):
    _module_name = "library/files"

    def create(
        self,
        file_path: str,
        *,
        path: Optional[str] = None,
        labels: Optional[List[str]] = None,
        public_url: Optional[str] = None,
        **kwargs,
    ) -> str:
        url = f"{self._client.get_base_url()}/{self._module_name}"
        files = {"file": open(file_path, "rb")}
        body = {"path": path, "labels": labels, "publicUrl": public_url}

        raw_response = self._post(url=url, files=files, body=body)

        return raw_response["fileId"]

    def get(self, file_id: str) -> FileResponse:
        url = f"{self._client.get_base_url()}/{self._module_name}/{file_id}"
        raw_response = self._get(url=url)

        return FileResponse.from_dict(raw_response)

    def list(
        self,
        *,
        offset: Optional[int] = None,
        limit: Optional[int] = None,
        **kwargs,
    ) -> List[FileResponse]:
        url = f"{self._client.get_base_url()}/{self._module_name}"
        params = {"offset": offset, "limit": limit}
        raw_response = self._get(url=url, params=params)

        return [FileResponse.from_dict(file) for file in raw_response]

    def update(
        self,
        file_id: str,
        *,
        public_url: Optional[str] = None,
        labels: Optional[List[str]] = None,
        **kwargs,
    ) -> None:
        url = f"{self._client.get_base_url()}/{self._module_name}/{file_id}"
        body = {
            "publicUrl": public_url,
            "labels": labels,
        }
        self._put(url=url, body=body)

    def delete(self, file_id: str) -> None:
        url = f"{self._client.get_base_url()}/{self._module_name}/{file_id}"
        self._delete(url=url)


class LibrarySearch(StudioResource):
    _module_name = "library/search"

    def create(
        self,
        query: str,
        *,
        path: Optional[str] = None,
        field_ids: Optional[List[str]] = None,
        max_segments: Optional[int] = None,
        **kwargs,
    ) -> LibrarySearchResponse:
        url = f"{self._client.get_base_url()}/{self._module_name}"
        body = {
            "query": query,
            "path": path,
            "fieldIds": field_ids,
            "maxSegments": max_segments,
        }
        raw_response = self._post(url=url, body=body)
        return LibrarySearchResponse.from_dict(raw_response)


class LibraryAnswer(StudioResource):
    _module_name = "library/answer"

    def create(
        self,
        question: str,
        *,
        path: Optional[str] = None,
        field_ids: Optional[List[str]] = None,
        labels: Optional[List[str]] = None,
        answer_length: Optional[AnswerLength] = None,
        mode: Optional[Mode] = None,
        **kwargs,
    ) -> LibraryAnswerResponse:
        url = f"{self._client.get_base_url()}/{self._module_name}"
        body = {
            "question": question,
            "path": path,
            "fieldIds": field_ids,
            "labels": labels,
            "answerLength": answer_length,
            "mode": mode,
        }
        raw_response = self._post(url=url, body=body)
        return LibraryAnswerResponse.from_dict(raw_response)



from ai21.clients.common.gec_base import GEC
from ai21.models.responses.gec_response import GECResponse
from ai21.clients.sagemaker.resources.sagemaker_resource import SageMakerResource


class SageMakerGEC(SageMakerResource, GEC):
    def create(self, text: str, **kwargs) -> GECResponse:
        body = self._create_body(text=text)

        response = self._invoke(body)

        return self._json_to_response(response)



from typing import List, Optional

from ai21.clients.common.chat_base import Chat
from ai21.clients.studio.resources.studio_resource import StudioResource
from ai21.models.chat_message import ChatMessage
from ai21.models.penalty import Penalty
from ai21.models.responses.chat_response import ChatResponse


class StudioChat(StudioResource, Chat):
    def create(
        self,
        model: str,
        messages: List[ChatMessage],
        system: str,
        *,
        num_results: Optional[int] = 1,
        temperature: Optional[float] = 0.7,
        max_tokens: Optional[int] = 300,
        min_tokens: Optional[int] = 0,
        top_p: Optional[float] = 1.0,
        top_k_return: Optional[int] = 0,
        stop_sequences: Optional[List[str]] = None,
        frequency_penalty: Optional[Penalty] = None,
        presence_penalty: Optional[Penalty] = None,
        count_penalty: Optional[Penalty] = None,
        **kwargs,
    ) -> ChatResponse:
        body = self._create_body(
            model=model,
            messages=messages,
            system=system,
            num_results=num_results,
            temperature=temperature,
            max_tokens=max_tokens,
            min_tokens=min_tokens,
            top_p=top_p,
            top_k_return=top_k_return,
            stop_sequences=stop_sequences,
            frequency_penalty=frequency_penalty,
            presence_penalty=presence_penalty,
            count_penalty=count_penalty,
        )
        url = f"{self._client.get_base_url()}/{model}/{self._module_name}"
        response = self._post(url=url, body=body)
        return self._json_to_response(response)



from dataclasses import dataclass
from typing import List

from ai21.models.ai21_base_model_mixin import AI21BaseModelMixin


@dataclass
class Suggestion(AI21BaseModelMixin):
    text: str


@dataclass
class ParaphraseResponse(AI21BaseModelMixin):
    id: str
    suggestions: List[Suggestion]



from dataclasses import dataclass
from typing import Optional

from ai21.models.ai21_base_model_mixin import AI21BaseModelMixin


@dataclass
class BaseModelMetadata(AI21BaseModelMixin):
    learning_rate: float
    num_epochs: int
    default_epoch: int
    cost: Optional[float] = None
    validation_loss: Optional[float] = None
    eval_loss: Optional[float] = None


@dataclass
class CustomBaseModelResponse(AI21BaseModelMixin):
    id: str
    name: str
    tier: str
    model_type: str
    custom_model_type: Optional[str]
    status: str
    model_metadata: BaseModelMetadata
    dataset_id: int
    dataset_name: str
    creation_date: str
    current_epoch: Optional[int]
    size: Optional[str] = None



from typing import List

from ai21.clients.common.improvements_base import Improvements
from ai21.clients.studio.resources.studio_resource import StudioResource
from ai21.errors import EmptyMandatoryListError
from ai21.models import ImprovementType
from ai21.models.responses.improvement_response import ImprovementsResponse


class StudioImprovements(StudioResource, Improvements):
    def create(self, text: str, types: List[ImprovementType], **kwargs) -> ImprovementsResponse:
        if len(types) == 0:
            raise EmptyMandatoryListError("types")

        url = f"{self._client.get_base_url()}/{self._module_name}"
        body = self._create_body(text=text, types=types)
        response = self._post(url=url, body=body)

        return self._json_to_response(response)



from abc import ABC, abstractmethod
from typing import Optional, Any, Dict

from ai21.models.document_type import DocumentType
from ai21.models.responses.summarize_by_segment_response import (
    SummarizeBySegmentResponse,
)


class SummarizeBySegment(ABC):
    _module_name = "summarize-by-segment"

    @abstractmethod
    def create(
        self,
        source: str,
        source_type: DocumentType,
        *,
        focus: Optional[str] = None,
        **kwargs,
    ) -> SummarizeBySegmentResponse:
        pass

    def _json_to_response(self, json: Dict[str, Any]) -> SummarizeBySegmentResponse:
        return SummarizeBySegmentResponse.from_dict(json)

    def _create_body(
        self,
        source: str,
        source_type: str,
        focus: Optional[str],
    ) -> Dict[str, Any]:
        return {
            "source": source,
            "sourceType": source_type,
            "focus": focus,
        }



from dataclasses import dataclass
from typing import List

from ai21.models.ai21_base_model_mixin import AI21BaseModelMixin


@dataclass
class EmbedResult(AI21BaseModelMixin):
    embedding: List[float]


@dataclass
class EmbedResponse(AI21BaseModelMixin):
    id: str
    results: List[EmbedResult]



from typing import Optional, Any, Dict

from ai21.ai21_env_config import _AI21EnvConfig, AI21EnvConfig
from ai21.ai21_http_client import AI21HTTPClient
from ai21.clients.studio.resources.studio_answer import StudioAnswer
from ai21.clients.studio.resources.studio_chat import StudioChat
from ai21.clients.studio.resources.studio_completion import StudioCompletion
from ai21.clients.studio.resources.studio_custom_model import StudioCustomModel
from ai21.clients.studio.resources.studio_dataset import StudioDataset
from ai21.clients.studio.resources.studio_embed import StudioEmbed
from ai21.clients.studio.resources.studio_gec import StudioGEC
from ai21.clients.studio.resources.studio_improvements import StudioImprovements
from ai21.clients.studio.resources.studio_library import StudioLibrary
from ai21.clients.studio.resources.studio_paraphrase import StudioParaphrase
from ai21.clients.studio.resources.studio_segmentation import StudioSegmentation
from ai21.clients.studio.resources.studio_summarize import StudioSummarize
from ai21.clients.studio.resources.studio_summarize_by_segment import StudioSummarizeBySegment
from ai21.http_client import HttpClient
from ai21.tokenizers.ai21_tokenizer import AI21Tokenizer
from ai21.tokenizers.factory import get_tokenizer


class AI21Client:
    """
    These class would be sending requests to our REST API using http requests
    """

    _tokenizer: Optional[AI21Tokenizer]

    def __init__(
        self,
        api_key: Optional[str] = None,
        api_host: Optional[str] = None,
        headers: Optional[Dict[str, Any]] = None,
        timeout_sec: Optional[float] = None,
        num_retries: Optional[int] = None,
        via: Optional[str] = None,
        http_client: Optional[HttpClient] = None,
        env_config: _AI21EnvConfig = AI21EnvConfig,
        **kwargs,
    ):
        self._http_client = AI21HTTPClient(
            api_key=api_key or env_config.api_key,
            api_host=api_host or env_config.api_host,
            api_version=env_config.api_version,
            headers=headers,
            timeout_sec=timeout_sec or env_config.timeout_sec,
            num_retries=num_retries or env_config.num_retries,
            via=via,
            http_client=http_client,
        )
        self.completion = StudioCompletion(self._http_client)
        self.chat = StudioChat(self._http_client)
        self.summarize = StudioSummarize(self._http_client)
        self.embed = StudioEmbed(self._http_client)
        self.gec = StudioGEC(self._http_client)
        self.improvements = StudioImprovements(self._http_client)
        self.paraphrase = StudioParaphrase(self._http_client)
        self.summarize_by_segment = StudioSummarizeBySegment(self._http_client)
        self.custom_model = StudioCustomModel(self._http_client)
        self.dataset = StudioDataset(self._http_client)
        self.answer = StudioAnswer(self._http_client)
        self.library = StudioLibrary(self._http_client)
        self.segmentation = StudioSegmentation(self._http_client)

    def count_tokens(self, text: str) -> int:
        # We might want to cache the tokenizer instance within the class
        # and not globally as it might be used by other instances

        tokenizer = get_tokenizer()
        return tokenizer.count_tokens(text)



import json
import re
from typing import Any, Dict

import boto3
from botocore.exceptions import ClientError

from ai21.logger import logger
from ai21.errors import AccessDenied, NotFound, APITimeoutError
from ai21.http_client import handle_non_success_response

_ERROR_MSG_TEMPLATE = (
    r"Received client error \((.*?)\) from primary with message \"(.*?)\". "
    r"See .* in account .* for more information."
)
_RUNTIME_NAME = "bedrock-runtime"


class BedrockSession:
    def __init__(self, session: boto3.Session, region: str):
        self._session = session.client(_RUNTIME_NAME) if session else boto3.client(_RUNTIME_NAME, region_name=region)

    def invoke_model(self, model_id: str, input_json: str) -> Dict[str, Any]:
        from botocore.exceptions import ClientError

        try:
            response = self._session.invoke_model(
                modelId=model_id,
                contentType="application/json",
                accept="application/json",
                body=input_json,
            )

            return json.loads(response.get("body").read())

        except ClientError as client_error:
            self._handle_client_error(client_error)
        except Exception as exception:
            logger.error(f"Calling {model_id} failed with Exception: {exception}")
            raise exception

    def _handle_client_error(self, client_exception: ClientError) -> None:
        error_response = client_exception.response
        error_message = error_response.get("Error", {}).get("Message", "")
        status_code = error_response.get("ResponseMetadata", {}).get("HTTPStatusCode", None)
        # As written in https://docs.aws.amazon.com/bedrock/latest/APIReference/API_runtime_InvokeModel.html

        if status_code == 403:
            raise AccessDenied(details=error_message)

        if status_code == 404:
            raise NotFound(details=error_message)

        if status_code == 408:
            raise APITimeoutError(details=error_message)

        if status_code == 424:
            error_message_template = re.compile(_ERROR_MSG_TEMPLATE)
            model_status_code = int(error_message_template.search(error_message).group(1))
            model_error_message = error_message_template.search(error_message).group(2)
            handle_non_success_response(model_status_code, model_error_message)

        handle_non_success_response(status_code, error_message)



from typing import List, Optional

from ai21.clients.common.embed_base import Embed
from ai21.clients.studio.resources.studio_resource import StudioResource
from ai21.models.responses.embed_response import EmbedResponse


class StudioEmbed(StudioResource, Embed):
    def create(self, texts: List[str], type: Optional[str] = None, **kwargs) -> EmbedResponse:
        url = f"{self._client.get_base_url()}/{self._module_name}"
        body = self._create_body(texts=texts, type=type)
        response = self._post(url=url, body=body)

        return self._json_to_response(response)



from typing import Optional

import boto3

from ai21.ai21_env_config import AI21EnvConfig, _AI21EnvConfig
from ai21.clients.bedrock.bedrock_session import BedrockSession
from ai21.clients.bedrock.resources.bedrock_completion import BedrockCompletion


class AI21BedrockClient:
    """
    :param session: An optional boto3 session to use for the client.
    """

    def __init__(
        self,
        session: Optional[boto3.Session] = None,
        region: Optional[str] = None,
        env_config: _AI21EnvConfig = AI21EnvConfig,
    ):

        self._bedrock_session = BedrockSession(session=session, region=region or env_config.aws_region)
        self.completion = BedrockCompletion(self._bedrock_session)



from ai21.models.answer_length import AnswerLength
from ai21.models.chat_message import ChatMessage
from ai21.models.document_type import DocumentType
from ai21.models.embed_type import EmbedType
from ai21.models.improvement_type import ImprovementType
from ai21.models.mode import Mode
from ai21.models.paraphrase_style_type import ParaphraseStyleType
from ai21.models.penalty import Penalty
from ai21.models.responses.answer_response import AnswerResponse
from ai21.models.responses.chat_response import ChatResponse, ChatOutput, FinishReason
from ai21.models.responses.completion_response import (
    CompletionsResponse,
    Completion,
    CompletionFinishReason,
    CompletionData,
    Prompt,
)
from ai21.models.responses.custom_model_response import CustomBaseModelResponse, BaseModelMetadata
from ai21.models.responses.dataset_response import DatasetResponse
from ai21.models.responses.embed_response import EmbedResponse, EmbedResult
from ai21.models.responses.file_response import FileResponse
from ai21.models.responses.gec_response import GECResponse, Correction, CorrectionType
from ai21.models.responses.improvement_response import ImprovementsResponse, Improvement
from ai21.models.responses.library_answer_response import LibraryAnswerResponse, SourceDocument
from ai21.models.responses.library_search_response import LibrarySearchResponse, LibrarySearchResult
from ai21.models.responses.paraphrase_response import ParaphraseResponse, Suggestion
from ai21.models.responses.segmentation_response import SegmentationResponse
from ai21.models.responses.summarize_by_segment_response import SummarizeBySegmentResponse, SegmentSummary, Highlight
from ai21.models.responses.summarize_response import SummarizeResponse
from ai21.models.role_type import RoleType
from ai21.models.summary_method import SummaryMethod


__all__ = [
    "AnswerLength",
    "Mode",
    "ChatMessage",
    "RoleType",
    "Penalty",
    "EmbedType",
    "ImprovementType",
    "ParaphraseStyleType",
    "DocumentType",
    "SummaryMethod",
    "AnswerResponse",
    "ChatResponse",
    "ChatOutput",
    "FinishReason",
    "CompletionsResponse",
    "Completion",
    "CompletionFinishReason",
    "CompletionData",
    "Prompt",
    "CustomBaseModelResponse",
    "BaseModelMetadata",
    "DatasetResponse",
    "EmbedResponse",
    "EmbedResult",
    "FileResponse",
    "GECResponse",
    "Correction",
    "CorrectionType",
    "ImprovementsResponse",
    "Improvement",
    "LibraryAnswerResponse",
    "SourceDocument",
    "LibrarySearchResponse",
    "LibrarySearchResult",
    "ParaphraseResponse",
    "Suggestion",
    "SegmentationResponse",
    "SegmentSummary",
    "Highlight",
    "SummarizeBySegmentResponse",
    "SummarizeResponse",
]



from dataclasses import dataclass
from typing import Optional

from ai21.models.ai21_base_model_mixin import AI21BaseModelMixin


@dataclass
class Penalty(AI21BaseModelMixin):
    scale: float
    apply_to_whitespaces: Optional[bool] = None
    apply_to_punctuation: Optional[bool] = None
    apply_to_numbers: Optional[bool] = None
    apply_to_stopwords: Optional[bool] = None
    apply_to_emojis: Optional[bool] = None



import json
import re

import boto3
from botocore.exceptions import ClientError

from ai21.errors import BadRequest, ServiceUnavailable, AI21ServerError, AI21APIError
from ai21.http_client import handle_non_success_response
from ai21.logger import logger

_ERROR_MSG_TEMPLATE = (
    r"Received client error \((.*?)\) from primary with message \"(.*?)\". "
    r"See .* in account .* for more information."
)
_SAGEMAKER_RUNTIME_NAME = "sagemaker-runtime"


class SageMakerSession:
    def __init__(self, session: boto3.Session, region: str, endpoint_name: str):
        self._session = session if session else boto3.client(_SAGEMAKER_RUNTIME_NAME, region_name=region)
        self._region = region
        self._endpoint_name = endpoint_name

    def invoke_endpoint(
        self,
        input_json: str,
    ):
        try:
            response = self._session.invoke_endpoint(
                EndpointName=self._endpoint_name,
                ContentType="application/json",
                Accept="application/json",
                Body=input_json,
            )

            return json.load(response["Body"])
        except ClientError as sm_client_error:
            self._handle_client_error(sm_client_error)
        except Exception as exception:
            logger.error(f"Calling {self._endpoint_name} failed with Exception: {exception}")
            raise exception

    def _handle_client_error(self, client_exception: "ClientError"):
        error_response = client_exception.response
        error_message = error_response.get("Error", {}).get("Message", "")
        status_code = error_response.get("ResponseMetadata", {}).get("HTTPStatusCode", None)
        # According to https://docs.aws.amazon.com/sagemaker/latest/APIReference/API_runtime_InvokeEndpoint.html#API_runtime_InvokeEndpoint_Errors
        if status_code == 400:
            raise BadRequest(details=error_message)
        if status_code == 424:
            error_message_template = re.compile(_ERROR_MSG_TEMPLATE)
            model_status_code = int(error_message_template.search(error_message).group(1))
            model_error_message = error_message_template.search(error_message).group(2)
            handle_non_success_response(model_status_code, model_error_message)
        if status_code == 429 or status_code == 503:
            raise ServiceUnavailable(details=error_message)
        if status_code == 500:
            raise AI21ServerError(details=error_message)
        raise AI21APIError(status_code, details=error_message)
